
; ---------------------------------------------------------------------------
; NewTestBed.asm
; ---------------------------------------------------------------------
; Use a Console Screen Format in DOS style.
; Convert it into a CHAR_INFO structure. 
; INVOKE the API: WriteConsoleOutput() to display it all at once . 
; 
; Check for Windows version and display it. 
; Check the CPU brand and display it
; Run a test on a number of Algos, and display results.
; Wait a key to terminate the program, to repeat the test, to save
;      result or to read a previous saved result.
;  
; Only the following keys are accepted.
;
; ESC - Key    Exit the program without asking to confirm
; R/r - Key    Rerun the test - Not Ready
; S/s - Key    Save the actual Screen to a POP File with the chosen name
; V/v - Key    Read and Display a previous saved Screen in POP File
; F1  - Key    Display a Help Screen
; H/h - Key    the same as F1 - another Help Screen
; D/d - Key    Switch between 12/24 Algos Display Mode - Not Ready 
; C/c - Key    Copy Text Screen to Win Clipboard - Not Ready
; E/e - Key    prompt a dialog box to confirm the exit
;
;----------------------------------------------------------------------
; Input file: Precompiled DB generated by Bin2dbx
;             POP files read from Disk
;----------------------------------------------------------------------
; Author: frktons @ MASM32 forum
; Date: 11/nov/2010. 
;---------------------------------------------------------------------- 
; In this version:
;
; the sizes of the screen and the console are read from the first 4 bytes,
; indicating RowInitial, RowFinal, ColInitial, ColFinal.
; 
; The S/s and V/v options are implemented to Save the actual Screen and
; to Read any previous saved one. 
;
; MsgBox used to display messages to the user.
;
; Input File as INCLUDE - not optimized yet for size reduction.
;
; Open File / Save File Dialog to choose Screen File to View / Save.
;
; F1/H Display Help Screens.
; 
; ESC is used to exit immediately the program
;
; The E/e option pop up a box to confirm exiting the program
;
; The D/d option is used to switch between 12/24 Display Mode
; 
;----------------------------------------------------------------------
; Enjoy - Frank
;----------------------------------------------------------------------------


include \masm32\include\masm32rt.inc

.686
.mmx
.xmm

include \masm32\macros\timers.asm
			
; get them from the 
; [url=http://www.masm32.com/board/index.php?topic=770.0]Masm32 Laboratory[/url]

LOOP_COUNT	= 100000 ; Number of iterations to perform

ustrv$ MACRO DDvalue1, DDvalue2   ;; unsigned integer to string
     invoke crt__ultoa,DDvalue1,DDvalue2,10
     EXITM <offset DDvalue2>
ENDM

;----------------------------------------------------------------------------
; PROC PROTO
;----------------------------------------------------------------------------

DisplayFmt     PROTO :DWORD
ClearBuffer    PROTO :DWORD
GetWindowsOS   PROTO :PTR BYTE
DisplayAt      PROTO x:DWORD, y:DWORD, StringBuffer:DWORD, len:DWORD
SetColorTo     PROTO ForeGroundColor:DWORD, BackGroundColor:DWORD
ConvertToWin   PROTO :DWORD, :DWORD, :DWORD
ReadChosenFile PROTO :DWORD, :DWORD 
PopBox         PROTO PtrBoxTitle:DWORD, PtrBoxMessage:DWORD 
SaveScreen     PROTO :DWORD
SaveChosenFile PROTO :DWORD, :DWORD
AxCPUid        PROTO :DWORD
FormatNum      PROTO :DWORD
AssignStr      PROTO PtrDest:DWORD, PtrSource:DWORD, MaxLen:DWORD
AlexConvertArea      PROTO  :DWORD, :DWORD,  :DWORD
SaveArea       PROTO :DWORD, :WORD, :WORD, :WORD, :WORD 
DisplayArea    PROTO :DWORD, :WORD, :WORD, :WORD, :WORD
 

;----------------------------------------------------------------------------
; Numeric constants used by the program
;----------------------------------------------------------------------------

Zero          EQU 0                  ;initial position on the buffer
One           EQU 1
Two           EQU 2                  ;To get the buffer size from chars num
Four          EQU 4                  ;number of initial bytes to read
Eight         EQU 8
N16           EQU 16

DM12          EQU 12
DM24          EQU 24 

MaxRows       EQU 34                 ;Max Rows of Console Display Mode 12
MaxCols       EQU 90                 ;Max Cols of Console Display Mode 24

MaxRows24     EQU 27                 ;Max Rows of Console
MaxCols24     EQU 90                 ;Max Cols of Console

CharNumber    EQU MaxRows * MaxCols
CharNumber24  EQU MaxRows24 * MaxCols24

NumCycles     EQU MaxRows * MaxCols / Two
NumCycles24   EQU MaxRows24 * MaxCols24 / Two

BytesToRead   EQU CharNumber * Two
BytesToRead24 EQU CharNumber24 * Two

MaxSize       EQU MaxRows * MaxCols * Four      ;Max buffer size Display Mode 12
MaxSize24     EQU MaxRows24 * MaxCols24 * Four  ;Max buffer size Display Mode 24


TestAreaMaxCols  EQU  43
TestAreaMaxRows  EQU  23

TestAreaRowInit  EQU  Eight
TestAreaRowEnd   EQU  30
TestAreaColInit  EQU  46
TestAreaColEnd   EQU  88

MaxLenCpuDesc    EQU  80

RowCpuDesc1      EQU  Two
RowCpuDesc2      EQU  3
ColCpuDesc       EQU  7

ColAlgoToTest    EQU  ColCpuDesc

TestAreaChars    EQU  TestAreaMaxCols * TestAreaMaxRows

ColTicksDisplacement   EQU 11 ; 
ColTicksInit           EQU 46 ;   

PRODUCT_PROFESSIONAL EQU 30h ; defined in WinNT.h Windows SDK 7.0A

; ----------------------------------------------------------------
;  Console code colors  
; ----------------------------------------------------------------


DarkBlue     EQU One

DarkBlack    EQU Zero
DarkBlue     EQU One
DarkGreen    EQU Two
DarkCyan     EQU 3
DarkRed      EQU 4
DarkMagenta  EQU 5
DarkBrown    EQU 6
DarkWhite    EQU 7


BrightGray    EQU 8
BrightBlue    EQU 9
BrightGreen   EQU 10
BrightCyan    EQU 11
BrightRed     EQU 12
BrightMagenta EQU 13
BrightYellow  EQU 14
BrightWhite   EQU 15

;----------------------------------------------------------------


; ----------------------------------------------------------------
;  Keyboard keys   
; ----------------------------------------------------------------

ArrowUp     EQU 72
ArrowDown   EQU 80
ArrowRight  EQU 77
ArrowLeft   EQU 75
PagDown     EQU 81
PagUp       EQU 73
Home        EQU 71
EndKey      EQU 79
EnterKey    EQU 13        




R_Key       EQU 82
r_Key       EQU 114
S_Key       EQU 83
s_Key       EQU 115
V_Key       EQU 86
v_Key       EQU 118
C_Key       EQU 67 
c_Key       EQU 99
D_Key       EQU 68 
d_Key       EQU 100
E_Key       EQU 69
e_Key       EQU 101
H_Key       EQU 72
h_Key       EQU 104
I_Key       EQU 73
i_Key       EQU 105

OK          EQU 6
YES         EQU One
NO          EQU Zero


;----------------------------------------------------------------------------
; Initialized data
;----------------------------------------------------------------------------

.data


    ConTitle  db "  ----------     **  New Testbed  ** General Screen File "   
              db "Display Tool  **      ---------- Ver. RC1", 0   



    AlgoToTest    db   "Conversion of a screen buffer from"
                  db   " DOS to Windows CHAR_INFO structure",0  


    BoxTitle    DB  "Program Message", 0
    BoxMessage  DB  "Sorry friend,",13,10,"This option is not",13,10
                DB  "Implemented yet, try in a few days",0 

    BoxMsgSaved DB  "Screen saved to Disk",0   

    BoxMsgErrSave DB "Error while saving File to Disk",0               

    BoxMsgErrFour DB  "Error while saving initial 4 bytes to Disk",0 

    BoxMsgBack    DB  "Back from the operation",0

    BoxMsgStart    DB  "Start the operation",0

    RunAgain      DWORD Zero


      
    ExitMessage DB  "Confirm exit the program ?",0
    ExitTitle   DB  "Exit Program",0   

    CaptionRead   BYTE  "Open a Screen File to Display",0

    CaptionSave   BYTE  "Save a Screen File to Disk",0
    
    szFilter      BYTE  "POP files *.pop",0,"*.pop",0,0

    BufFileBytes  dd   BytesToRead  ;buffer size to read the POP file

    DisplayMode   dd   DM12 ; Initially the Dispaly Mode is set to 12 Algos

;---------------------------------------------------------------------------------

    Sep         DB  ",",0   ;   used for decimal numbers separator - choose yours
    lpSep       DD  Sep     ;   When calling GetNumberFormat
    Tsep        DD  ".",0   ;   used for thousand numbers separator - choose yours
    lpTsep      DD  Tsep    ;   When calling GetNumberFormat

;---------------------------------------------------------------------------------

    PtrBufNum        DD BufNum



        
    AlgoDescSize     DWORD SIZEOF AlgoDesc
    

    
    AlgoSizeStr      BYTE  9 DUP (?)
    PtrAlgoSizeStr   DWORD AlgoSizeStr
    AlgoSizeStrLen   DWORD ?
    PtrAlgoSizeStrLen DWORD ?
    AlgoTicks        DWORD ?
    AlgoTicksStr     BYTE  9 DUP (?)
    PtrAlgoTicksStr  DWORD AlgoTicksStr    
    AlgoRow          DWORD  8
    AlgoColDesc      DWORD  4
    AlgoColSize      DWORD  40
    AlgoColTicks     DWORD  46
    StringLenght     DWORD ?
    TestToDo         DWORD ?
    TestDone         DWORD 0

    MaxLenStr        DWORD 32

    CpuDesc1         BYTE 81 dup (0)
    CpuDesc2         BYTE 81 dup (0)  





;----------------------------------------------------------------------------
; Include Screen to display - precompiled db for Main Screen Display Mode 12
;----------------------------------------------------------------------------  

include NewTestBed12.inc

;----------------------------------------------------------------------------
; Include Screen to display - precompiled db for Test Area Display Mode 24
;----------------------------------------------------------------------------  

include NewTestBed24.inc
    
          
;----------------------------------------------------------------------------
; Uninitialized data
;----------------------------------------------------------------------------         

.data?

    ALIGN N16

    ConsoleScreen12 CHAR_INFO CharNumber dup (<>)

    SecurityArea  db   N16 dup (?)

    ALIGN N16

    ConsoleScreen24 CHAR_INFO CharNumber24 dup (<>)

    SecurityAreax db   N16 dup (?)

    ALIGN N16

    SavedScreen24 CHAR_INFO CharNumber24 dup (<>)

    SecurityAreay db   N16 dup (?)

    ALIGN N16

    FileScreen    CHAR_INFO CharNumber dup (<>)

    SecurityArea0 db   N16 dup (?)

    ALIGN N16

    SavedScreen   CHAR_INFO CharNumber dup (<>)

    SecurityArea1 db   N16 dup (?)

    ALIGN N16

    CleanTestArea CHAR_INFO CharNumber dup (<>)   

    SecurityArea2 db   N16 dup (?)

    ALIGN N16

    ResultScreen  CHAR_INFO CharNumber dup (<>)   

    SecurityArea3 db   N16 dup (?)

    ALIGN N16

    HelpScreen    CHAR_INFO CharNumber dup (<>)   

    SecurityArea4 db   N16 dup (?)

    ALIGN N16

    TestAreaBuffer CHAR_INFO TestAreaChars dup (<>) 

    SecurityArea5 db   N16 dup (?)



    RowInitialFile    db   ?    
    RowFinalFile      db   ?    
    ColInitialFile    db   ?   
    ColFinalFile      db   ?  

    OddNumber     dd   ?

    ChosenColor   dd   ?
    ActualColor   dd   ? 

    Lenght        dd   ?
    Lenght1       dd   ?
    Lenght2       dd   ?        
        
  

    R1            dw   ?
    R2            dw   ?
    C1            dw   ?
    C2            dw   ?
    

    BufRows       dw   ?          ;buffer height (rows)
    BufCols       dw   ?          ;buffer width (columns)
    BufChars      dd   ?          ;number of chars in the buffer

    FileBuffer2   db   BytesToRead  dup (?)
    HelpBuffer    db   BytesToRead  dup (?)    


    PtrFileName   DWORD ?   

    hFile     HANDLE ? 
    cci       CONSOLE_CURSOR_INFO <>        


    wHnd      HANDLE ? 
    rHnd      HANDLE ?

    windowSize SMALL_RECT <> ; Structures used to Read
    bufferSize COORD      <> ; and Write to the Screen
    startPoint COORD      <> ; the whole screen

    TestAreaWindowSize SMALL_RECT <> ; Structures used to Read
    TestAreaBufferSize COORD      <> ; and Write to the Screen
    TestAreaStartPoint COORD      <> ; the Test Area Screen 

    

    howmany dd ?
    buffer  INPUT_RECORD <>

    OSVERSIONINFOEX_NEW STRUCT
      dwOSVersionInfoSize   DWORD ?
      dwMajorVersion        DWORD ?
      dwMinorVersion        DWORD ?
      dwBuildNumber         DWORD ?
      dwPlatformId          DWORD ?
      szCSDVersion          BYTE  128  dup (?)
      wServicePackMajor     WORD  ?
      wServicePackMinor     WORD  ?
      wSuiteMask            WORD  ?
      wProductType          BYTE  ?
      wReserved             BYTE  ?
    OSVERSIONINFOEX_NEW ENDS

    CTEXT MACRO text:VARARG
      LOCAL TxtName
      .DATA
        TxtName BYTE text,0
      .CODE
      EXITM <ADDR TxtName>
    ENDM

    szOS   BYTE   256 dup (?)

    CpuBuffer BYTE 260 dup (?)
    pbuf   DWORD ?

    PtrCpuBuf2   DWORD ?

  

    ConfirmExit      dd ?
    ButtonClicked    dd ?

    DisplayFlag      dd ?

    TextString       BYTE 64 dup (?)
    PtrTextString    dd ?

    AlgoNum          dd ?

    BufNum           DB  255 dup(?)     ; used by GetNumberFormat
    NumFmt           NUMBERFMT <>       ; to format with thousand separator integer 
                                        ; numbers



;---------------------------------------------------------------------------------
; Structure for number formatting
;---------------------------------------------------------------------------------   
; NUMBERFMTA STRUCT
;    NumDigits         DWORD      ?
;    LeadingZero       DWORD      ?
;    Grouping          DWORD      ?
;    lpDecimalSep      DWORD      ?
;    lpThousandSep     DWORD      ?
;    NegativeOrder     DWORD      ?
; NUMBERFMTA ENDS
;
; NUMBERFMT  equ  <NUMBERFMTA>     
;---------------------------------------------------------------------------------    



.code

start: 

; ---------------------------------------------------------------------------
; The Main Proc shows the program flow.
; ---------------------------------------------------------------------------

Main PROC

    mov RowInitialFile, One    
    mov RowFinalFile,   MaxRows    
    mov ColInitialFile, One   
    mov ColFinalFile,   MaxCols  

;----------------------------------------------------------------------------------------------------
; Initialize structure for calling GetNumberFormat
;----------------------------------------------------------------------------------------------------

mov NumFmt.NumDigits, 0               ; 0 for No decimal digits
mov NumFmt.LeadingZero, 0             ; 0 = No leading zeroes
mov NumFmt.Grouping, 3                ; Groups of three numbers
mov NumFmt.lpDecimalSep,  offset Sep  ; Pointer to NULL terminated string with the decimal separator 
mov NumFmt.lpThousandSep, offset Tsep ; Pointer to NULL terminated string with the thousand separator
mov NumFmt.NegativeOrder, 0           ; Negative number mode OFF

;----------------------------------------------------------------------------------------------------

    mov    DisplayFlag, NO
    
    CALL   ConsoleSize 

; -------------------------------------------------
;  Not needed so far
; -------------------------------------------------
;
;    INVOKE ClearBuffer, ADDR ConsoleScreen12
;
;    INVOKE ClearBuffer, ADDR FileScreen
;
;    INVOKE ClearBuffer, ADDR SavedScreen
;
;    INVOKE ClearBuffer, ADDR ResultScreen
;
;    INVOKE ClearBuffer, ADDR HelpScreen
;
; -------------------------------------------------

    CALL   GetConsole

    CALL   InitProc

    mov    PtrTextString, offset  TextString

    mov    PtrTextString, add$(PtrTextString, "TESTBEDHELP.POP") 

    INVOKE ReadChosenFile, DWORD PTR PtrTextString, ADDR HelpBuffer

    INVOKE AlexConvertArea, ADDR HelpScreen, ADDR HelpBuffer, NumCycles

    CALL   ConvertXmm

    INVOKE ConvertToWin, ADDR ConsoleScreen24, ADDR FileBuffer24, NumCycles24

    INVOKE DisplayFmt,   offset ConsoleScreen12

    INVOKE SaveArea, offset TestAreaBuffer, TestAreaRowInit, TestAreaColInit, 
                     TestAreaRowEnd, TestAreaColEnd
                      
    
    mov    ActualColor,  Zero
    
    INVOKE GetWindowsOS, ADDR szOS
    
    INVOKE lstrcat, ADDR szOS, CTEXT(13,10); Eliminate duplicate spaces

    INVOKE SetColorTo, BrightYellow, DarkBlue

    mov    Lenght, len(ADDR szOS)
   
    INVOKE DisplayAt, 7, 1, ADDR szOS, DWORD PTR Lenght

; ###########################################################################
; # This code is addeded by Alex 
; ###########################################################################
      
    invoke AxCPUid, offset CpuBuffer
    
    ; -----------------------------------------------------------------------
    ; NOTE: After a call to ^^^ this proc, the EAX is constructed, read
    ; comments above proc
    ; -----------------------------------------------------------------------
        
    movzx ebx,ax	; maximal instruction sets, supported by CPU

    ; -------------------------------------------------------------------------------
    ; for example, if test include code, which is required SSSE3, then need to check
    ; an AX (or EBX in this sample) for 5. If number is smaller than 5, then SSSE3 
    ; is not supported and needed to manage that thing.
    ; -------------------------------------------------------------------------------
    
    
    

    ; -------------------------------------------------------------------------------
    ; This part is exactly the type of coding I'm trying to avoid
    ; -------------------------------------------------------------------------------
    ; mov esi, offset CpuBuffer
    ; invoke szMonoSpace, esi
    ;
    ; invoke lstrlen,esi
    ; push eax
    ; push esi
    ;
    ; push 2	; y
    ; push 7	; x
    ; call DisplayAt
    ; -------------------------------------------------------------------------------    
    
; ###########################################################################
;  Code replaced by Alex - he likes it popped and pushed :-) / I don't. ;-)
; ###########################################################################

    lea    esi, CpuBuffer

    mov    pbuf, esi

    invoke szMonoSpace, esi                ; format it

; -------------------------------------------------------------------------------
; Test a long Cpu description
; -------------------------------------------------------------------------------
;
;    .data
;
;    Temp  db  "Intel(R) Core(TM)2 CPU 6600 @ 2.40GHz with MMX, SSE1, SSE2, SSE3," 
;          db  " SSSE3, SSE4.1, SSE4.2 and some more stuff",0
;
;    .code
;
;    mov    Lenght, len(ADDR Temp)
;
;    INVOKE AssignStr, ADDR CpuBuffer, ADDR Temp, 
;                      DWORD PTR Lenght
;
;  
;
;    jmp    TestDisplay
;
; -------------------------------------------------------------------------------
; End of Test a long Cpu description
; -------------------------------------------------------------------------------
    
    mov    Lenght, len(ADDR CpuBuffer)

TestDisplay:

    .IF ( Lenght <= MaxLenCpuDesc )
   
        INVOKE DisplayAt, ColCpuDesc, RowCpuDesc1, ADDR CpuBuffer, DWORD PTR Lenght ; display it  

    .ELSE

        CALL   SplitCpuDesc

        INVOKE DisplayAt, ColCpuDesc, RowCpuDesc1, ADDR CpuDesc1, DWORD PTR Lenght1

        INVOKE DisplayAt, ColCpuDesc, RowCpuDesc2, ADDR CpuDesc2, DWORD PTR Lenght2                   


    .ENDIF

; ###########################################################################

    mov    Lenght, len(ADDR AlgoToTest)

    INVOKE DisplayAt, ColAlgoToTest, Four, ADDR AlgoToTest, DWORD PTR Lenght



; ###########################################################################
; ---------------------------------------------------------------------------
;  First Algo to test - Uses MMX opcodes / Alex Version
; ---------------------------------------------------------------------------
; ###########################################################################


    .data

    AlgoSize        DWORD EndAlgo1 - Algo1

    AlgoDesc        BYTE  "Alex / MMX - PUNPCKLBW MOVQ ",0


    .code


    .IF DisplayMode == DM12

        mov  TestToDo, Four

    .ELSE

        mov  TestToDo, One

    .ENDIF

    INVOKE DisplayAt, DWORD PTR AlgoColDesc, DWORD PTR AlgoRow, ADDR AlgoDesc, 
                      DWORD PTR AlgoDescSize

    .IF ( AlgoSize > 99 )

        sub  AlgoColSize, One

    .ENDIF

    mov PtrAlgoSizeStr, ustrv$(AlgoSize, offset AlgoSizeStr)

    mov AlgoSizeStrLen, len(PtrAlgoSizeStr) 

    INVOKE DisplayAt, DWORD PTR AlgoColSize, DWORD PTR AlgoRow, ADDR AlgoSizeStr, 
                      DWORD PTR AlgoSizeStrLen 

    .IF ( AlgoSize > 99 )

        add  AlgoColSize, One

    .ENDIF

StartAlgo1:

    invoke Sleep, 100
    counter_begin LOOP_COUNT, HIGH_PRIORITY_CLASS

Algo1:

    lea  esi, FileBuffer
    lea  edi, ConsoleScreen12
    
    mov  ecx, CharNumber
    mov  eax, CharNumber
    pxor mm7,mm7
    and  ecx,1
    shr  eax,1
    jz   @no2
    
    @@:
    
    movd mm0,dword ptr [esi]
    punpcklbw mm0,mm7
    add  esi,  4
    movq qword ptr [edi],mm0
    add  edi,  8
    add  eax, -1
    jnz  @B
    
    @no2:
    
    jecxz @done
    movzx eax,word ptr [esi]
    mov   [edi],al
    mov   [edi+2],ah   
        
    @done:
    
    emms    

EndAlgo1:

    counter_end

    mov AlgoTicks, eax 

    mov PtrAlgoTicksStr, ustrv$(AlgoTicks, offset AlgoTicksStr)

    .IF (AlgoTicks > 999)

       INVOKE FormatNum, PtrAlgoTicksStr;

       mov    StringLenght, len (PtrBufNum)

       .IF  ( StringLenght < 9 )

            mov  eax, 9
            
            sub  eax, [StringLenght]

            add  AlgoColTicks, eax
            
       .ENDIF 

       INVOKE DisplayAt, DWORD PTR AlgoColTicks, DWORD PTR AlgoRow, ADDR BufNum, 
                         DWORD PTR StringLenght

       jmp NextTest1  


    .ENDIF

     

    INVOKE DisplayAt, DWORD PTR AlgoColTicks, DWORD PTR AlgoRow, ADDR AlgoTicksStr, 
                      SIZEOF AlgoTicksStr 



NextTest1: 

    inc    TestDone

    mov    eax, TestToDo

    .IF TestDone >= eax

        mov  TestDone, Zero

        jmp  EndTestAlgo1

    .ENDIF

    .IF  ( StringLenght < 9 )

         mov  eax, 9
            
         sub  eax, [StringLenght]

         sub  AlgoColTicks, eax
            
    .ENDIF 

    add AlgoColTicks, ColTicksDisplacement

    jmp StartAlgo1

EndTestAlgo1:

    mov AlgoColTicks, ColTicksInit

    add AlgoRow, Two

; ###########################################################################
; ---------------------------------------------------------------------------
;  Second Algo to test - Uses 386 opcodes / Frank Version
; ---------------------------------------------------------------------------
; ###########################################################################


    .data  

    AlgoDesc2       BYTE  "Frank / 486 - MOV-BSWAP",0

    AlgoSize2       DWORD EndAlgo2 - Algo2

    .code

    .IF ( RunAgain == YES )

        jmp StartAlgo2

    .ENDIF

    INVOKE AssignStr, ADDR AlgoDesc, ADDR AlgoDesc2, 
                      DWORD PTR MaxLenStr


    INVOKE DisplayAt, DWORD PTR AlgoColDesc, DWORD PTR AlgoRow, ADDR AlgoDesc, 
                      DWORD PTR AlgoDescSize

    mov eax, AlgoSize2
    mov AlgoSize, eax

    .IF ( AlgoSize > 99 )

        sub  AlgoColSize, One

    .ENDIF

    mov PtrAlgoSizeStr, ustrv$(AlgoSize, offset AlgoSizeStr)
    
    mov AlgoSizeStrLen, len(PtrAlgoSizeStr) 

    INVOKE DisplayAt, DWORD PTR AlgoColSize, DWORD PTR AlgoRow, ADDR AlgoSizeStr, 
                      DWORD PTR AlgoSizeStrLen 

    .IF ( AlgoSize > 99 )

        add  AlgoColSize, One

    .ENDIF


StartAlgo2:

    invoke Sleep, 100
    counter_begin LOOP_COUNT, HIGH_PRIORITY_CLASS

Algo2:

    lea  esi, FileBuffer
    lea  edi, ConsoleScreen12

    mov  ecx, NumCycles
    xor  eax, eax

NextChar2:


    mov eax, [esi]
    mov [edi], al
    mov [edi+Two], ah

    bswap eax

    mov [edi+Four], ah    
    mov [edi+6], al

    add esi, Four
    add edi, Four*Two

    dec ecx
    jnz NextChar2 


EndAlgo2:

    counter_end

    mov AlgoTicks, eax 

    mov PtrAlgoTicksStr, ustrv$(AlgoTicks, offset AlgoTicksStr)

    .IF (AlgoTicks > 999)

       INVOKE FormatNum, PtrAlgoTicksStr;

       mov    StringLenght, len (PtrBufNum)

       .IF  ( StringLenght < 9 )

            mov  eax, 9
            
            sub  eax, [StringLenght]

            add  AlgoColTicks, eax
            
       .ENDIF 

       INVOKE DisplayAt, DWORD PTR AlgoColTicks, DWORD PTR AlgoRow, ADDR BufNum, 
                         DWORD PTR StringLenght

       jmp NextTest2  


    .ENDIF

     

    INVOKE DisplayAt, DWORD PTR AlgoColTicks, DWORD PTR AlgoRow, ADDR AlgoTicksStr, 
                      SIZEOF AlgoTicksStr 



NextTest2: 

    inc    TestDone

    mov    eax, TestToDo

    .IF TestDone >= eax

        mov  TestDone, Zero

        jmp  EndTestAlgo2

    .ENDIF

    .IF  ( StringLenght < 9 )

         mov  eax, 9
            
         sub  eax, [StringLenght]

         sub  AlgoColTicks, eax
            
    .ENDIF 

    add AlgoColTicks, ColTicksDisplacement

    jmp StartAlgo2

EndTestAlgo2:

    mov AlgoColTicks, ColTicksInit

    add AlgoRow, Two

; ###########################################################################
; ---------------------------------------------------------------------------
;  Third Algo to test - Uses XMM opcodes / Frank Version
; ---------------------------------------------------------------------------
; ###########################################################################


    .data

    AlgoSize3        DWORD EndAlgo3 - Algo3

    AlgoDesc3        BYTE  "Frank / XMM PUNPCKLBW MOVDQA",0


    .code

    .IF ( RunAgain == YES )

        jmp StartAlgo3

    .ENDIF


    INVOKE AssignStr, ADDR AlgoDesc, ADDR AlgoDesc3, 
                      DWORD PTR MaxLenStr


    .IF DisplayMode == DM12

        mov  TestToDo, Four

    .ELSE

        mov  TestToDo, One

    .ENDIF

    INVOKE DisplayAt, DWORD PTR AlgoColDesc, DWORD PTR AlgoRow, ADDR AlgoDesc, 
                      DWORD PTR AlgoDescSize

    mov eax, AlgoSize3
    mov AlgoSize, eax

    .IF ( AlgoSize > 99 )

        sub  AlgoColSize, One

    .ENDIF

    mov PtrAlgoSizeStr, ustrv$(AlgoSize, offset AlgoSizeStr)

    mov AlgoSizeStrLen, len(PtrAlgoSizeStr) 

    INVOKE DisplayAt, DWORD PTR AlgoColSize, DWORD PTR AlgoRow, ADDR AlgoSizeStr, 
                      DWORD PTR AlgoSizeStrLen 

    .IF ( AlgoSize > 99 )

        add  AlgoColSize, One

    .ENDIF

StartAlgo3:

    invoke Sleep, 100
    counter_begin LOOP_COUNT, HIGH_PRIORITY_CLASS

Algo3:

    lea  esi, FileBuffer
    lea  edi, ConsoleScreen12
    
    mov  ecx, NumCycles
    pxor xmm7,xmm7

    shr  ecx,1
    
@@:
    
    movq      xmm0, qword ptr [esi]
    punpcklbw xmm0, xmm7
    add       esi,  Eight
    movdqa    [edi],xmm0
    add       edi, N16
    dec       ecx
    jnz       @B 

EndAlgo3:

    counter_end

    mov AlgoTicks, eax 

    mov PtrAlgoTicksStr, ustrv$(AlgoTicks, offset AlgoTicksStr)

    .IF (AlgoTicks > 999)

       INVOKE FormatNum, PtrAlgoTicksStr;

       mov    StringLenght, len (PtrBufNum)

       .IF  ( StringLenght < 9 )

            mov  eax, 9
            
            sub  eax, [StringLenght]

            add  AlgoColTicks, eax
            
       .ENDIF 

       INVOKE DisplayAt, DWORD PTR AlgoColTicks, DWORD PTR AlgoRow, ADDR BufNum, 
                         DWORD PTR StringLenght

       jmp NextTest3  


    .ENDIF

     

    INVOKE DisplayAt, DWORD PTR AlgoColTicks, DWORD PTR AlgoRow, ADDR AlgoTicksStr, 
                      SIZEOF AlgoTicksStr 



NextTest3: 

    inc    TestDone

    mov    eax, TestToDo

    .IF TestDone >= eax

        mov  TestDone, Zero

        jmp  EndTestAlgo3

    .ENDIF

    .IF  ( StringLenght < 9 )

         mov  eax, 9
            
         sub  eax, [StringLenght]

         sub  AlgoColTicks, eax
            
    .ENDIF 

    add AlgoColTicks, ColTicksDisplacement

    jmp StartAlgo3

EndTestAlgo3:

    mov AlgoColTicks, ColTicksInit

    add AlgoRow, Two





; ---------------------------------------------------------------------------

    mov    ConfirmExit, NO

    INVOKE SaveScreen, offset SavedScreen

    mov RunAgain, NO

      
    CALL   AnyKey

    .IF  ( ConfirmExit == YES )

        jmp Finish

    .ENDIF

    .IF  ( RunAgain == YES )

        jmp StartAlgo1

    .ENDIF

    CALL   ShowTheCursor

Finish: 

    exit

Main ENDP


;----------------------------------------------------------------------------------------------------
; Procedure for copying a string to another for max lenght specified
;----------------------------------------------------------------------------------------------------

AssignStr  PROC PtrDest:DWORD, PtrSource:DWORD, MaxLen:DWORD

  mov   edi, PtrDest
  mov   esi, PtrSource
  mov   ecx, MaxLen
  xor   ebx, ebx 

  .WHILE (ebx != ecx)

       mov    al, BYTE PTR [esi]
       mov [edi], al
       
       .BREAK  .IF ( al == 0)

       inc    esi
       inc    edi

       inc    ebx

  .ENDW 

  xor  edx, edx
    
  mov  [edi], dl  

  ret 


AssignStr  ENDP

;----------------------------------------------------------------------------------------------------
; Procedure for formatting a string with thousand comma/point separator
;----------------------------------------------------------------------------------------------------


FormatNum PROC PtrNumber:DWORD

  invoke GetNumberFormat, NULL, 0, PtrNumber, offset NumFmt, offset BufNum, 255

  ret  

FormatNum ENDP

;----------------------------------------------------------------------------------------------------

 
; ---------------------------------------------------------------------------
; The size, title and buffer of the Console are set.
; Max Console Size is set to 34 Rows and 90 Cols.
; ---------------------------------------------------------------------------

ConsoleSize PROC

    INVOKE AllocConsole

    INVOKE GetStdHandle, STD_INPUT_HANDLE
    mov rHnd,eax

    INVOKE GetStdHandle, STD_OUTPUT_HANDLE
    mov wHnd,eax 

    INVOKE SetConsoleTitle, ADDR ConTitle
  
    CALL   HideTheCursor

    mov windowSize.Left, Zero
    mov windowSize.Right, (MaxCols -1)
    mov windowSize.Top, Zero
    mov windowSize.Bottom, (MaxRows -1)

    
    mov bufferSize.x, MaxCols
    mov bufferSize.y, MaxRows

    INVOKE SetConsoleWindowInfo, wHnd, TRUE, ADDR windowSize

    INVOKE SetConsoleScreenBufferSize, wHnd, DWORD PTR bufferSize

    ret

ConsoleSize ENDP

; ---------------------------------------------------------------------------
; Reads the Screen File Corner Positions.
; ---------------------------------------------------------------------------
;    The initial 4 bytes contain:
;
;    RowInitial, RowFinal, ColInitial, ColFinal of the Screen File to display
; ---------------------------------------------------------------------------
 

GetConsole PROC 


    movzx eax, RowFinal
    mov   R2,  ax
    inc   eax
    sub   al,  RowInitial

    mov   BufRows, ax

    movzx ebx, ColFinal
    mov   C2,  bx    
    inc   ebx
    sub   bl,  ColInitial

    mov   BufCols, bx

    movzx ax, RowInitial  
    mov   R1, ax

    dec   R1
    dec   R2
    movzx ax, ColInitial
    mov   C1, ax 
    dec   C1   
    dec   C2

    mov   ebx, BytesToRead

    mov   BufChars, ebx

    ret 
    
GetConsole ENDP

;----------------------------------------------------------------------------
; The Cursor is made visible
;----------------------------------------------------------------------------

ShowTheCursor PROC

    INVOKE GetConsoleCursorInfo,wHnd,addr cci
    mov cci.bVisible,TRUE
    INVOKE SetConsoleCursorInfo,wHnd,addr cci
    ret
    
ShowTheCursor ENDP

;----------------------------------------------------------------------------
; The Cursor is made invisible
;----------------------------------------------------------------------------
    


HideTheCursor PROC

    INVOKE GetConsoleCursorInfo,wHnd,addr cci
    mov cci.bVisible,FALSE
    INVOKE SetConsoleCursorInfo,wHnd,addr cci
    ret
    
HideTheCursor ENDP

;----------------------------------------------------------------------------
; After the Screen File has been read, the parameters to display it are set
;----------------------------------------------------------------------------

InitProc PROC

 
    mov ax, C1
    mov windowSize.Left, ax
    mov ax, C2
    mov windowSize.Right, ax
    mov ax, R1
    mov windowSize.Top, ax
    mov ax, R2
    mov windowSize.Bottom, ax
   
    mov ax, BufRows
    mov bx, BufCols
    
    mov bufferSize.x, bx
    mov bufferSize.y, ax

    mov startPoint.x, Zero
    mov startPoint.y, Zero


    ret

InitProc ENDP

;----------------------------------------------------------------------------
; The Screen File is displayed on the Console 
;----------------------------------------------------------------------------

DisplayFmt PROC szFmtName:DWORD 

    INVOKE WriteConsoleOutput, wHnd, szFmtName, DWORD PTR bufferSize, 
           DWORD PTR startPoint, offset windowSize 

    ret

DisplayFmt ENDP

;----------------------------------------------------------------------------
; The Screen File is saved for future use 
;----------------------------------------------------------------------------

SaveScreen PROC szFmtName:DWORD 

    INVOKE ReadConsoleOutput, wHnd, szFmtName, DWORD PTR bufferSize, 
           DWORD PTR startPoint, offset windowSize 

    ret

SaveScreen ENDP


;----------------------------------------------------------------------------
; Area of the Screen is saved for future use 
;----------------------------------------------------------------------------

SaveArea PROC szFmtName:DWORD, RowStart:WORD, ColStart:WORD,
                               RowEnd:WORD,   ColEnd:WORD 


; ---------------------------------------------------------------------------

    mov ax, ColStart
    mov TestAreaWindowSize.Left, ax

    mov ax, RowStart
    mov TestAreaWindowSize.Top, ax

    mov ax, ColEnd
    mov TestAreaWindowSize.Right, ax

    mov ax, RowEnd
    mov TestAreaWindowSize.Bottom, ax

    
    mov TestAreaBufferSize.x, TestAreaMaxCols
    mov TestAreaBufferSize.y, TestAreaMaxRows

    INVOKE ReadConsoleOutput, wHnd, szFmtName, DWORD PTR TestAreaBufferSize, 
           DWORD PTR startPoint, offset TestAreaWindowSize 

    ret

SaveArea ENDP

;----------------------------------------------------------------------------
; Area of the Screen is displayed  
;----------------------------------------------------------------------------

DisplayArea PROC szFmtName:DWORD, RowStart:WORD, ColStart:WORD,
                                  RowEnd:WORD,   ColEnd:WORD 


; ---------------------------------------------------------------------------

    mov ax, ColStart
    mov TestAreaWindowSize.Left, ax

    mov ax, RowStart
    mov TestAreaWindowSize.Top, ax

    mov ax, ColEnd
    mov TestAreaWindowSize.Right, ax

    mov ax, RowEnd
    mov TestAreaWindowSize.Bottom, ax

    
    mov TestAreaBufferSize.x, TestAreaMaxCols
    mov TestAreaBufferSize.y, TestAreaMaxRows

    INVOKE WriteConsoleOutput, wHnd, szFmtName, DWORD PTR TestAreaBufferSize, 
           DWORD PTR startPoint, offset TestAreaWindowSize 

    ret

DisplayArea ENDP

; ----------------------------------------------------------------------------
;  The Cpu Description string is splitted in two parts if Lenght > 80
; ----------------------------------------------------------------------------


SplitCpuDesc   PROC

    lea esi, CpuBuffer
    add esi, ( MaxLenCpuDesc - One )
    mov Lenght1, MaxLenCpuDesc
    mov Lenght2, Zero  

    mov al, BYTE PTR [esi]  

    .WHILE ( al != "," )

        dec esi
        dec Lenght1  
        mov edi, [esi] 
        
    .ENDW

    lea esi, CpuBuffer
    lea edi, CpuDesc1

    mov ecx, Lenght1

CopyChar1:

    rep movs BYTE PTR [edi], BYTE PTR [esi]

    inc esi

    mov PtrCpuBuf2, esi

    mov eax, ( Lenght )
    sub eax, Lenght1
    mov Lenght2, eax

    lea edi, CpuDesc2  

    mov ecx, Lenght2


CopyChar2:

    rep movs BYTE PTR [edi], BYTE PTR [esi]

    ret

SplitCpuDesc   ENDP

; ----------------------------------------------------------------------------
;  The program waits for a key pressed
; ----------------------------------------------------------------------------
;  Returns: key code in buffer.KeyEvent.wVirtualKeyCode WORD size 
; -------------------------------------------------------------------------
;  In final release to recode with Switch Block Syntax for easy maintenance
;  and reading.
; -------------------------------------------------------------------------
;     switch uMsg
;      case WM_INITDIALOG
;         invoke SendMessage,hWin,WM_SETICON,1, FUNC(LoadIcon,NULL,IDI_ASTERISK)
;         m2m hWnd, hWin
;         return 1
;     
;       case WM_COMMAND
;         switch wParam
;           case IDOK
;             fn MessageBox,hWnd,"You clicked OK","Hi",MB_OK or MB_ICONINFORMATION
;     
;           case IDCANCEL
;             jmp quit_dialog
;         endsw
;     
;       case WM_CLOSE
;         quit_dialog:
;         invoke EndDialog,hWin,0
;     endsw
; -------------------------------------------------------------------------
;     switch variable
;       case value1 ; there must be at least one (1) case statement
;         ; do something here
;       case value2 ; optional extra case statement(s)
;         ; do something else
;       default ; optional default processing
;         ; do any default processing
;     endsw
;     
; ----------------------------------------------------------------------------------


AnyKey PROC

again:  

    INVOKE ReadConsoleInput,rHnd,offset buffer,1,offset howmany
    cmp buffer.EventType,KEY_EVENT
    jnz again

    cmp buffer.KeyEvent.bKeyDown,0
    jz again

    mov ax, buffer.KeyEvent.wVirtualKeyCode

    cmp al, E_Key
    jz  ClosePgm

    cmp al, e_Key
    jz  ClosePgm

    .IF ( al == VK_ESCAPE )

        mov ConfirmExit, YES

        jmp  BackToMain

    .ENDIF
    

    .IF ( al == R_Key || al == r_Key )

        mov RunAgain, YES

        mov AlgoColTicks, ColTicksInit

        mov  AlgoRow, Eight

        INVOKE DisplayArea, offset TestAreaBuffer, TestAreaRowInit, TestAreaColInit, 
                            TestAreaRowEnd, TestAreaColEnd

        jmp  BackToMain

    .ENDIF


    cmp al, S_Key
    jz  S_Key_Press

    cmp al, s_Key
    jz  S_Key_Press

    cmp al, V_Key
    jz  V_Key_Press

    cmp al, v_Key
    jz  V_Key_Press

    cmp al, C_Key
    jz  C_Key_Press

    cmp al, c_Key
    jz  C_Key_Press

    cmp al, D_Key
    jz  D_Key_Press

    cmp al, d_Key
    jz  D_Key_Press

    cmp al, H_Key
    jz  F1_Key_Press

    cmp al, h_Key
    jz  F1_Key_Press

    cmp al, I_Key
    jz  I_Key_Press

    cmp al, i_Key
    jz  I_Key_Press

    cmp al, VK_F1
    jz  F1_Key_Press

    jmp again


C_Key_Press:

    INVOKE PopBox,  OFFSET BoxTitle, OFFSET BoxMessage 

    jmp again

D_Key_Press:

    INVOKE PopBox,  OFFSET BoxTitle, OFFSET BoxMessage 

    jmp again
    
I_Key_Press:

    INVOKE PopBox,  OFFSET BoxTitle, OFFSET BoxMessage 

    jmp again

S_Key_Press:

    CALL   SaveScreenFile

    jmp again

F1_Key_Press:

    INVOKE SaveScreen, offset SavedScreen

    INVOKE DisplayFmt, offset HelpScreen

    jmp WaitingForAKey


; -------------------------------------------------------------
;    Mov DisplayFlag, NO
;
;    jmp WaitingForAKey
; -------------------------------------------------------------

V_Key_Press:

    CALL   ReadScreenFile

; -------------------------------------------------------------
;    mov    DisplayFlag, YES
; -------------------------------------------------------------

WaitingForAKey:  

    INVOKE ReadConsoleInput,rHnd,offset buffer,1,offset howmany
    cmp buffer.EventType,KEY_EVENT
    jnz WaitingForAKey

    cmp buffer.KeyEvent.bKeyDown,0
    jz WaitingForAKey

; -------------------------------------------------------------
;    cmp DisplayFlag, NO
;
;    jne  DisplaySaved
;
;    Mov DisplayFlag, YES   
;
;    jmp again
;
; -------------------------------------------------------------

DisplaySaved:

    INVOKE DisplayFmt, offset SavedScreen

    jmp again


ClosePgm:

    CALL PopBoxExit

    cmp  ConfirmExit, YES

    jnz  again

BackToMain:

    ret 

AnyKey ENDP

; -------------------------------------------------------------
; Wait for a key before continuing
; -------------------------------------------------------------

Waiting PROC  

StillWaiting:

    INVOKE ReadConsoleInput,rHnd,offset buffer,1,offset howmany
    cmp buffer.EventType,KEY_EVENT
    jnz StillWaiting

    cmp buffer.KeyEvent.bKeyDown,0
    jz StillWaiting
    
    ret 

Waiting ENDP

; -------------------------------------------------------------------------
; The data read from the file is converted into CHAR_INFO style. 
; The Console Buffer is filled with the content of the File Buffer, padding 
; a zero after each byte making it Windows Console API displayable. 
; -------------------------------------------------------------------------

ConvertToWin PROC DestBuffer:DWORD, SourceBuffer:DWORD, NumCyclesToDo:DWORD


    mov  esi, SourceBuffer
    mov  edi, DestBuffer

    mov  ecx, NumCyclesToDo
    xor  eax, eax

    xor  ebx, ebx

    bt   ecx, 0
    jc   IsOdd
    mov  OddNumber, 0
    jmp  NextChar

IsOdd:

    mov OddNumber, 1
    
NextChar:


    mov eax, [esi]
    mov [edi], al
    mov [edi+2], ah

    bswap eax

    mov [edi+Four], ah    
    mov [edi+6], al

    add esi, Four
    add edi, Four*2

    dec ecx
    jnz NextChar 

; --------------------------------------------------------------------------------
;  Verify if the number of characters is odd to manage the extra-bytes
; --------------------------------------------------------------------------------

    bt  OddNumber, 0
    jnc End_cycle

    movzx eax, WORD PTR [esi]
    mov [edi], al
    mov [edi+2], ah   

End_cycle:

    ret 
    
ConvertToWin ENDP

; --------------------------------------------------------------------------------
;  Convert the byte string into word string using XMM register and punpcklbw
; --------------------------------------------------------------------------------

ConvertXmm PROC

    lea  esi, FileBuffer
    lea  edi, ConsoleScreen12
    
    mov  ecx, NumCycles

    pxor xmm7,xmm7

    shr  ecx,1

    
    @@:
    
    movq      xmm0, qword ptr [esi]
    punpcklbw xmm0, xmm7
    add       esi,  Eight
    movdqa    [edi],xmm0
    add  edi, (Eight * Two)
    dec  ecx
    jnz  @B
    
    
    ret    

ConvertXmm ENDP


; -------------------------------------------------------------------------
; The data read from the Screen Buffer is converted into DOS style. 
; The New Buffer is filled with the content of the Screen Buffer, removing 
; a char after each byte making it DOS compatible. 
; -------------------------------------------------------------------------

ConvertToDOS PROC DestBuffer:DWORD, SourceBuffer:DWORD


    mov  esi, SourceBuffer
    mov  edi, DestBuffer

    mov  ecx, CharNumber
    xor  eax, eax

    xor  ebx, ebx

NextChar:

    mov eax, [esi]
    mov [edi], al
    shr eax, N16
    mov [edi+1], al

    add esi, Four
    add edi, Two

    dec ecx
    jnz NextChar 

End_cycle:

    ret 
    
ConvertToDOS ENDP

; --------------------------------------------------------------------------------
;  The buffer to be displayed is initially zeroed
; -------------------------------------------------------------------------   

ClearBuffer PROC AddrBuffer:DWORD


    mov edx, AddrBuffer
    lea ecx, [edx + MaxSize]
    xor eax, eax
    movd xmm0, eax
    pshufd xmm0, xmm0, 0
  @@:
    movdqa [edx], xmm0
    add edx, N16
    cmp edx, ecx
    jl @B

    

    ret

ClearBuffer ENDP

; --------------------------------------------------------------------------------
; Set the attribute colors for next PRINT/DISPLAY instruction
;----------------------------------------------------------------------------


SetColorTo PROC Fore:DWORD, Back:DWORD

    xor  eax, eax

    mov  eax, Back
    shl  eax, Four
    add  eax, Fore
    mov  ChosenColor, eax 

    invoke SetConsoleTextAttribute, wHnd, ChosenColor

    ret

SetColorTo ENDP

; --------------------------------------------------------------------------------
; A string is displayed at Column and Row for the passed Lenght
; ----------------------------------------------------------------------------

DisplayAt PROC x:DWORD, y:DWORD, StringBuffer:DWORD, lenght:DWORD
   
    LOCAL n_of_chars:DWORD
    LOCAL position:COORD
   
    mov eax, x
    mov position.x, ax
    mov eax, y
    mov position.y, ax
   
    invoke SetConsoleCursorPosition, wHnd, DWORD PTR position
    invoke WriteConsole, wHnd, StringBuffer, lenght, ADDR n_of_chars, NULL
   
    ret
    
DisplayAt ENDP


; ----------------------------------------------------------------------------


;----------------------------------------------------------------------------
; Pop UP a Message Box to display some stuff
;----------------------------------------------------------------------------

PopBox PROC PtrBoxTitle:DWORD, PtrBoxMessage:DWORD 


    invoke MessageBox, NULL, PtrBoxMessage,  PtrBoxTitle,
                       MB_OK  OR MB_SYSTEMMODAL OR MB_ICONINFORMATION 

    ret

PopBox ENDP

;----------------------------------------------------------------------------
; Pop UP a Message Box to confirm exiting the program
;----------------------------------------------------------------------------

PopBoxExit PROC


    invoke MessageBox, NULL, offset ExitMessage,  offset ExitTitle,
                       MB_OK  OR MB_SYSTEMMODAL OR MB_ICONQUESTION OR MB_YESNO 

    mov ButtonClicked, eax

    cmp ButtonClicked, OK

    jnz GoBack

    mov ConfirmExit, YES
    
GoBack:  

    ret

PopBoxExit ENDP

;----------------------------------------------------------------------------
; Choose a saved Screen File to Read/Display
;----------------------------------------------------------------------------

ReadScreenFile PROC

    mov PtrFileName, OpenFileDlg (0,0, addr CaptionRead, addr szFilter) 

    cmp BYTE PTR [eax], 0 ; test if 1st byte is zero [No selection made]

    je  Finish

    INVOKE ReadChosenFile, DWORD PTR PtrFileName, ADDR FileBuffer2

    INVOKE ConvertToWin, ADDR FileScreen, ADDR FileBuffer2, NumCycles

    INVOKE DisplayFmt, offset FileScreen

    INVOKE SetColorTo, BrightWhite, DarkRed

    lea    eax, TextString
    xor    ebx, ebx

    mov    [eax], ebx

    mov    PtrTextString, offset TextString

    mov    PtrTextString, add$(PtrTextString, " Saved Screen File - Press any key to Return to Main ")    

    INVOKE DisplayAt, 20, 32, ADDR TextString, 53  

Finish:

    ret

ReadScreenFile ENDP  

;----------------------------------------------------------------------------
; Choose a name to save actual Screen to a Disk File
;----------------------------------------------------------------------------

SaveScreenFile PROC

    INVOKE SaveScreen, OFFSET ResultScreen

    INVOKE ConvertToDOS, ADDR FileBuffer2, ADDR ResultScreen 

    mov PtrFileName, SaveFileDlg (0,0, addr CaptionSave, addr szFilter) 

    cmp BYTE PTR [eax], 0 ; test if 1st byte is zero [No selection made]

    je  NoSave

    INVOKE SaveChosenFile, DWORD PTR PtrFileName, OFFSET FileBuffer2

NoSave:

    ret

SaveScreenFile ENDP  

; ---------------------------------------------------------------------------
; Read the Screen File.
; ---------------------------------------------------------------------------
; 1] The initial 4 bytes contain:
;
;    RowInitial, RowFinal, ColInitial, ColFinal of the Screen File to display
;
; 2] The remaining bytes contain the pairs of Characters/Attributes of
;
;    the Screen File in DOS style.
; ---------------------------------------------------------------------------
 

ReadChosenFile PROC szFmtName:DWORD, PtrBuf:DWORD 

    mov hFile, fopen(szFmtName)

    mov ebx,   fread(hFile, ADDR RowInitialFile, Four)

    mov ebx,   fread(hFile, PtrBuf, BytesToRead)

    fclose hFile

    ret 
    
ReadChosenFile ENDP  

; ---------------------------------------------------------------------------
; Save the Screen to a disk File.
; ---------------------------------------------------------------------------
; 1] The initial 4 bytes contain:
;
;    RowInitial, RowFinal, ColInitial, ColFinal of the Screen File to display
;
; 2] The remaining bytes contain the pairs of Characters/Attributes of
;
;    the Screen File in DOS style.
; ---------------------------------------------------------------------------
 

SaveChosenFile PROC szFmtName:DWORD, PtrBuf:DWORD 

    mov hFile,  fcreate(szFmtName)

WriteFour:

    mov ebx,   fwrite(hFile, ADDR RowInitialFile, Four)

    cmp ebx,   Four

    je  WriteScreen

    INVOKE PopBox,  OFFSET BoxTitle, OFFSET BoxMsgErrFour 

    jmp EndSave  

WriteScreen:

    mov ebx,   fwrite(hFile, PtrBuf, BytesToRead)

    cmp ebx,   BytesToRead

    je  SaveOK

SaveError:

    INVOKE PopBox,  OFFSET BoxTitle, OFFSET BoxMsgErrSave 

    jmp EndSave   

SaveOK:

    INVOKE PopBox,  OFFSET BoxTitle, OFFSET BoxMsgSaved 
    
EndSave:

    fclose hFile

    ret 
    
SaveChosenFile ENDP  

; --------------------------------------------------------------------------------
; Check for Windows Version - GregL updated version to detect Win 7
; -------------------------------------------------------------------------

GetWindowsOS PROC pszOS:PTR BYTE

    LOCAL hMod:DWORD
    LOCAL dwType:DWORD
    LOCAL pGNSI:PTR
    LOCAL pGPI:PTR
    LOCAL osvi:OSVERSIONINFOEX_NEW
    LOCAL sysinfo:SYSTEM_INFO
    LOCAL szBuild[N16]:BYTE
    LOCAL szBuf[128]:BYTE
    
    INVOKE RtlZeroMemory, ADDR osvi, SIZEOF OSVERSIONINFOEX
    INVOKE RtlZeroMemory, ADDR sysinfo, SIZEOF SYSTEM_INFO
    mov osvi.dwOSVersionInfoSize, SIZEOF OSVERSIONINFOEX
    INVOKE GetVersionEx, ADDR osvi
    ; Call GetNativeSystemInfo if supported or GetSystemInfo otherwise.
    INVOKE GetModuleHandle, CTEXT("kernel32.dll")
    mov hMod, eax
    INVOKE GetProcAddress, hMod, CTEXT("GetNativeSystemInfo")
    mov pGNSI, eax
    .IF pGNSI != NULL
        lea eax, sysinfo
        push eax
        call pGNSI
    .ELSE
        INVOKE GetSystemInfo, ADDR sysinfo
    .ENDIF  
    
    .IF (osvi.dwPlatformId == VER_PLATFORM_WIN32_NT) && (osvi.dwMajorVersion > 3)
        ; This program does not support dwMajorVersion less than 4
        INVOKE lstrcpy, pszOS, CTEXT("Microsoft ")
        .IF (osvi.dwMajorVersion == 6)
            .IF (osvi.dwMinorVersion == 0)
                ; Windows Vista or Windows Server 2008
                .IF osvi.wProductType == VER_NT_WORKSTATION
                    INVOKE lstrcat, pszOS, CTEXT("Windows Vista ")
                .ELSE
                    INVOKE lstrcat, pszOS, CTEXT("Windows Server 2008")
                .ENDIF
            .ELSEIF (osvi.dwMinorVersion == 1)
                ; Windows 7 or Windows Server 2008 R2
                .IF osvi.wProductType == VER_NT_WORKSTATION
                    INVOKE lstrcat, pszOS, CTEXT("Windows 7 ")
                .ELSE
                    INVOKE lstrcat, pszOS, CTEXT("Windows Server 2008 R2")
                .ENDIF
            .ENDIF
            
            INVOKE GetProcAddress, hMod, CTEXT("GetProductInfo")
            mov pGPI, eax
            lea eax, dwType
            push eax
            push 0
            push 0
            push osvi.dwMinorVersion
            push osvi.dwMajorVersion
            call pGPI
            .IF dwType == PRODUCT_ULTIMATE
               INVOKE lstrcat, pszOS, CTEXT("Ultimate Edition")
            .ELSEIF dwType == PRODUCT_PROFESSIONAL
               INVOKE lstrcat, pszOS, CTEXT("Professional")               
            .ELSEIF dwType == PRODUCT_HOME_PREMIUM
               INVOKE lstrcat, pszOS, CTEXT("Home Premium Edition")
            .ELSEIF dwType == PRODUCT_HOME_BASIC
               INVOKE lstrcat, pszOS, CTEXT("Home Basic Edition")
            .ELSEIF dwType == PRODUCT_ENTERPRISE
               INVOKE lstrcat, pszOS, CTEXT("Enterprise Edition")
            .ELSEIF dwType == PRODUCT_BUSINESS
               INVOKE lstrcat, pszOS, CTEXT("Business Edition")
            .ELSEIF dwType == PRODUCT_STARTER
               INVOKE lstrcat, pszOS, CTEXT("Starter Edition")
            .ELSEIF dwType == PRODUCT_CLUSTER_SERVER
               INVOKE lstrcat, pszOS, CTEXT("Cluster Server Edition")
            .ELSEIF dwType == PRODUCT_DATACENTER_SERVER
               INVOKE lstrcat, pszOS, CTEXT("Datacenter Edition")
            .ELSEIF dwType == PRODUCT_DATACENTER_SERVER_CORE
               INVOKE lstrcat, pszOS, CTEXT("Datacenter Edition (core installation)")
            .ELSEIF dwType == PRODUCT_ENTERPRISE_SERVER
               INVOKE lstrcat, pszOS, CTEXT("Enterprise Edition")
            .ELSEIF dwType == PRODUCT_ENTERPRISE_SERVER_CORE
               INVOKE lstrcat, pszOS, CTEXT("Enterprise Edition (core installation)")
            .ELSEIF dwType == PRODUCT_ENTERPRISE_SERVER_IA64
               INVOKE lstrcat, pszOS, CTEXT("Enterprise Edition for Itanium-based Systems")
            .ELSEIF dwType == PRODUCT_SMALLBUSINESS_SERVER
               INVOKE lstrcat, pszOS, CTEXT("Small Business Server")
            .ELSEIF dwType == PRODUCT_SMALLBUSINESS_SERVER_PREMIUM
               INVOKE lstrcat, pszOS, CTEXT("Small Business Server Premium Edition")
            .ELSEIF dwType == PRODUCT_STANDARD_SERVER
               INVOKE lstrcat, pszOS, CTEXT("Standard Edition")
            .ELSEIF dwType == PRODUCT_STANDARD_SERVER_CORE
               INVOKE lstrcat, pszOS, CTEXT("Standard Edition (core installation)")
            .ELSEIF dwType == PRODUCT_WEB_SERVER
               INVOKE lstrcat, pszOS, CTEXT("Web Server Edition")
            .ENDIF    
            .IF sysinfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_AMD64
                INVOKE lstrcat, pszOS, CTEXT( ", 64-bit" )
            .ELSEIF sysinfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_INTEL
                INVOKE lstrcat, pszOS, CTEXT(", 32-bit")
            .ENDIF
        .ELSEIF (osvi.dwMajorVersion == 5) && (osvi.dwMinorVersion == 2)
            ; Windows Server 2003    
            INVOKE GetSystemMetrics, SM_SERVERR2
            .IF eax
                INVOKE lstrcat, pszOS, CTEXT("Windows Server 2003 R2, ")
            .ELSEIF osvi.wSuiteMask == VER_SUITE_STORAGE_SERVER    
                INVOKE lstrcat, pszOS, CTEXT( "Windows Storage Server 2003")
            .ELSEIF (osvi.wProductType == VER_NT_WORKSTATION) && (sysinfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_AMD64)
                INVOKE lstrcat, pszOS, CTEXT( "Windows XP Professional x64 Edition")
            .ELSE
                INVOKE lstrcat, pszOS, CTEXT("Windows Server 2003, ")
            .ENDIF
            .IF osvi.wProductType != VER_NT_WORKSTATION
                .IF sysinfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_IA64
                    .IF osvi.wSuiteMask & VER_SUITE_DATACENTER
                       INVOKE lstrcat, pszOS, CTEXT("Datacenter Edition for Itanium-based Systems")
                    .ELSEIF osvi.wSuiteMask & VER_SUITE_ENTERPRISE
                       INVOKE lstrcat, pszOS, CTEXT("Enterprise Edition for Itanium-based Systems")
                    .ENDIF   
                .ENDIF
            .ELSEIF sysinfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_AMD64
                .IF osvi.wSuiteMask & VER_SUITE_DATACENTER
                    INVOKE lstrcat, pszOS, CTEXT("Datacenter x64 Edition")
                .ELSEIF osvi.wSuiteMask & VER_SUITE_ENTERPRISE
                    INVOKE lstrcat, pszOS, CTEXT("Enterprise x64 Edition")
                .ELSE 
                    INVOKE lstrcat, pszOS, CTEXT("Standard x64 Edition")
                .ENDIF    
            .ELSE
                .IF osvi.wSuiteMask & VER_SUITE_COMPUTE_SERVER
                    INVOKE lstrcat, pszOS, CTEXT("Compute Cluster Edition")
                .ELSEIF osvi.wSuiteMask & VER_SUITE_DATACENTER
                    INVOKE lstrcat, pszOS, CTEXT("Datacenter Edition")
                .ELSEIF osvi.wSuiteMask & VER_SUITE_ENTERPRISE
                    INVOKE lstrcat, pszOS, CTEXT("Enterprise Edition")
                .ELSEIF osvi.wSuiteMask & VER_SUITE_BLADE
                    INVOKE lstrcat, pszOS, CTEXT("Web Edition")
                .ELSE 
                    INVOKE lstrcat, pszOS, CTEXT("Standard Edition")
                .ENDIF    
            .ENDIF
        .ELSEIF (osvi.dwMajorVersion == 5) && (osvi.dwMinorVersion == 1)
            ; Windows XP            
            INVOKE lstrcat, pszOS, CTEXT("Windows XP ")
            .IF osvi.wSuiteMask & VER_SUITE_PERSONAL
                INVOKE lstrcat, pszOS, CTEXT("Home Edition")
            .ELSE 
                INVOKE lstrcat, pszOS, CTEXT("Professional")
            .ENDIF    
        .ELSEIF (osvi.dwMajorVersion == 5) && (osvi.dwMinorVersion == 0)
            ; Windows 2000
            INVOKE lstrcat, pszOS, CTEXT("Windows 2000 ")
             .IF osvi.wProductType == VER_NT_WORKSTATION
                 INVOKE lstrcat, pszOS, CTEXT("Professional" )
             .ELSE
                 .IF osvi.wSuiteMask & VER_SUITE_DATACENTER
                     INVOKE lstrcat, pszOS, CTEXT("Datacenter Server" )
                 .ELSEIF osvi.wSuiteMask & VER_SUITE_ENTERPRISE
                     INVOKE lstrcat, pszOS, CTEXT("Advanced Server")
                 .ELSE 
                     INVOKE lstrcat, pszOS, CTEXT("Server")
                 .ENDIF  
             .ENDIF    
        .ELSEIF osvi.dwMajorVersion == 4
            ; Windows NT 4.0
            INVOKE lstrcat, pszOS, CTEXT("Windows NT 4.0")
        .ENDIF  
        ; Include service pack (if any) and build number.
        INVOKE lstrlen, ADDR osvi.szCSDVersion 
        .IF eax > 0
            INVOKE lstrcat, pszOS, CTEXT(" ")
            INVOKE lstrcat, pszOS, ADDR osvi.szCSDVersion
        .ENDIF
        INVOKE wsprintf, ADDR szBuild, CTEXT(" (build %d)"), osvi.dwBuildNumber
        INVOKE lstrcat, pszOS, ADDR szBuild
        xor eax, eax
    .ELSE
        INVOKE lstrcpy, pszOS, 
        CTEXT("This program does not support Windows versions less than Windows NT 4.0")
        mov eax, 1
    .ENDIF
    ret
GetWindowsOS ENDP

; --------------------------------------------------------------------------------


;#############################################################################
; Instructions detection code by Alex aka Antariy
;#############################################################################

OPTION PROLOGUE:NONE
OPTION EPILOGUE:NONE

AxCPUid_Features	struct DWORD
	Is486capable	dd	?
	IsP1		dd	?
	IsP1MMX		dd	?
	IsPPro		dd	?
	IsSSE1		dd	?
	IsSSE2		dd	?
	IsSSE3		dd	?
	IsSSSE3		dd	?
	IsSSE41		dd	?
	IsSSE42		dd	?
	BrandName	db	64 dup (?)
AxCPUid_Features	ends

AxCPUidCodeSizeStart EQU $

; ---------------------------------------------------------------------------------
; Return values:
; Not zero - if entire structure was filled
; 0 - if CPUID is not supported or not entire struct beed filled.
;     For clearness, check the Is486capable - if 0 then CPUID is not supported,
;     otherwise CPU family less than PIV, but structure are filled properly.
; ---------------------------------------------------------------------------------

align N16

AxCPUid_FillStructure	proc	lpAxCPUid_Features:PTR AxCPUid_Features
	.data?
		ifrunnedAndSupportCPUid	dd	?
	.code
	
	push ebp
	push ebx
	
	xor eax,eax
	mov ecx,sizeof AxCPUid_Features

	mov ebp,[esp+4+8]
	
	@@:
	mov [ebp+ecx-4].AxCPUid_Features.Is486capable,eax
	add ecx,-4
	jnz @B

	mov ebx,ifrunnedAndSupportCPUid
	dec ebx
	jz @done
	dec ebx
	jz @is486capable
	
	inc ifrunnedAndSupportCPUid
	
	pushfd
	pop ecx
	mov ebx,ecx
	xor ebx,200000h	; switch ID flag
	push ebx
	popfd		; change EFLAGS
	pushfd
	pop ebx
	xor ebx,ecx	; if previous ID bit was equal to current
	test ebx,200000h; then it would be dropped
	jz @done	; otherwise - it set
	
	inc ifrunnedAndSupportCPUid
	
	@is486capable:

	or [ebp].AxCPUid_Features.Is486capable,1
	
	xor eax,eax
	cpuid
	test eax,eax
	jz @done
	
	mov eax,1
	cpuid
	
	shr edx,5	; has RDTSC - so, this is PI at least
	adc [ebp].AxCPUid_Features.IsP1,0
	
	shr edx,2	; has PAE - so, PPro at least
	adc [ebp].AxCPUid_Features.IsPPro,0
	
	shr edx,17	; has MMX
	adc [ebp].AxCPUid_Features.IsP1MMX,0
	
	shr edx,2	; has SSE1
	adc [ebp].AxCPUid_Features.IsSSE1,0
	
	shr edx,1	; has SSE2
	adc [ebp].AxCPUid_Features.IsSSE2,0
	
	
	
	shr ecx,1	; has SSE3
	adc [ebp].AxCPUid_Features.IsSSE3,0
	
	shr ecx,9	; has SSSE3
	adc [ebp].AxCPUid_Features.IsSSSE3,0
	
	shr ecx,10	; has SSE4.1
	adc [ebp].AxCPUid_Features.IsSSE41,0
	
	shr ecx,1	; has SSE4.2
	adc [ebp].AxCPUid_Features.IsSSE42,0
	
	
	; get CPU brand name, if exist
	

	; fix for PIII, not SSE2 capable CPU cannot have brand name
	cmp [ebp].AxCPUid_Features.IsSSE2,0
	jz @done

	mov eax,80000000h
	cpuid
	add eax,eax	; check for zero and no any extended functions
	jz @done
	
	lea ebp,[ebp].AxCPUid_Features.BrandName
	cmp eax,8	; needed at least 80000004h function
	mov eax,0
	jb @done
	
	push esi
	mov esi,80000002h
	@@:
	mov eax,esi
	cpuid
	inc esi
	mov [ebp],eax
	mov [ebp+4],ebx
	mov [ebp+8],ecx
	mov [ebp+12],edx
	add ebp, N16
	cmp esi,80000005h
	jb @B		
	pop esi
	
	or eax,1	; in case of terminated brand string
	
	@done:
	pop ebx
	pop ebp
	ret 4
AxCPUid_FillStructure	endp


; Return values:
; 	low order WORD of EAX:
; 		instructions supported:
; 			0 - need upgrade
; 			above 0 - then supported:
; 			1 - MMX
; 			2 - SSE1
; 			3 - SSE2
;			4 - SSE3
; 			5 - SSSE3
; 			6 - SSE4.1
; 			7 - SSE4.2
;	high order WORD of EAX:
;		size of the output string in chars, without termination null


align N16

AxCPUid proc lpStrOut:DWORD

	push ebx
	push esi
	push edi
	
	invoke GetStdHandle,STD_OUTPUT_HANDLE
	xchg eax,ebx
	
	mov esi,[esp+12+4]
		
	invoke AxCPUid_FillStructure,esi
	cmp [esi].AxCPUid_Features.Is486capable,0
	jnz @F	
	
	push eax
	mov edx,esp
	
	push 0
	push edx
	push sizeof @@needupgrade
	push offset @@needupgrade
	push ebx
	call WriteFile
	pop ecx
	xor eax,eax
	jmp @done
	
	@@:
	
	mov edi,[esi].AxCPUid_Features.IsP1MMX
	add edi,[esi].AxCPUid_Features.IsSSE1
	add edi,[esi].AxCPUid_Features.IsSSE2
	add edi,[esi].AxCPUid_Features.IsSSE3
	add edi,[esi].AxCPUid_Features.IsSSSE3
	add edi,[esi].AxCPUid_Features.IsSSE41
	add edi,[esi].AxCPUid_Features.IsSSE42
		
	mov eax,[esi].AxCPUid_Features.IsSSE42
	lea eax,[eax*2+offset @@sse42]

	mov edx,[esi].AxCPUid_Features.IsSSE41
	lea edx,[edx*2+offset @@sse41]
	
	mov ecx,[esi].AxCPUid_Features.IsSSSE3
	lea ecx,[ecx*2+offset @@ssse3]
	
	push eax
	push edx
	push ecx
	
	mov eax,[esi].AxCPUid_Features.IsSSE3
	lea eax,[eax*2+offset @@sse3]
	
	mov edx,[esi].AxCPUid_Features.IsSSE2
	lea edx,[edx*2+offset @@sse2]
	
	mov ecx,[esi].AxCPUid_Features.IsSSE1
	lea ecx,[ecx*2+offset @@sse1]
	
	push eax
	push edx
	push ecx	
	
	mov eax,[esi].AxCPUid_Features.IsP1MMX
	lea eax,[eax*2+offset @@mmx]
	
	lea edx,[esi].AxCPUid_Features.BrandName
	cmp dword ptr [edx],0
	jnz @hasbrandname

	mov ecx,[esi].AxCPUid_Features.Is486capable
	add ecx,[esi].AxCPUid_Features.IsP1
	add ecx,[esi].AxCPUid_Features.IsP1MMX
	add ecx,[esi].AxCPUid_Features.IsPPro
	
	cmp ecx,4
	jb @F
	add ecx,[esi].AxCPUid_Features.IsP1MMX	; PII is PPro with MMX
	add ecx,[esi].AxCPUid_Features.IsSSE1	; PIII is PPro with MMX and SSE1
	@@:
		
	mov edx,[ecx*4+offset @@earlycpus]
		
	@hasbrandname:
	add edx,1
	cmp byte ptr [edx-1]," "
	jz @hasbrandname
	
	dec edx
		
	push eax
	push edx
	push offset @@fmtFeatures
	push esi
	call wsprintf
	
	shl eax,16	; put count of chars into high word

	add esp,10*4

	add eax,edi	; put instructions set supported into AX
	@done:
	pop edi
	pop esi
	pop ebx
	ret 4
	
	even
	@@needupgrade	db	"This is time for upgrade indeed, i386 or early i486 :)"
	even
	@@i486		db	"Old-good 80486",0
	even
	@@p1		db	"Pentium",0
	even
	@@pmmx		db	"Pentium with MMX Technology",0
	even
	@@ppro		db	"Pentium Pro",0
	even
	@@p2		db	"Pentium II",0
	even
	@@p3		db	"Pentium III",0
	
	align 4
	@@earlycpus	dd	0
			dd	offset @@i486
			dd	offset @@p1
			dd	offset @@pmmx
			dd	offset @@ppro
			dd	offset @@p2
			dd	offset @@p3
	
	even
	@@fmtFeatures	db	"%s with %s%s%s%s%s%s%s",0
	even
	@@mmx		db	0,0
			db	"MMX",0
	even
	@@sse1		db	0,0
			db	", SSE1",0
	even
	@@sse2		db	0,0
			db	", SSE2",0
	even
	@@sse3		db	0,0
			db	", SSE3",0
	even
	@@ssse3		db	0,0
			db	", SSSE3",0
	even
	@@sse41		db	0,0
			db	", SSE4.1",0
	even
	@@sse42		db	0,0
			db	", SSE4.2",0
	
	
AxCPUid endp

OPTION PROLOGUE:PrologueDef
OPTION EPILOGUE:EpilogueDef
;#############################################################################

; -------------------------------------------------------------------------
; The data read from the file is converted into CHAR_INFO style. 
; ConsoleBuffer is filled with the content of FileBuffer, padding a zero
; after each byte making it Windows Console API displayable. 
; Alex version with mmx instructions and punpcklbw.
; -------------------------------------------------------------------------

 AlexConvertArea PROC DestBuffer:DWORD, SourceBuffer:DWORD, NumCyclesToDo:DWORD

    mov  esi, SourceBuffer
    mov  edi, DestBuffer
    
    mov  ecx, NumCyclesToDo
    mov  eax, NumCyclesToDo
    pxor mm7,mm7
    and ecx,1
    
    @@:
    
    movd mm0,dword ptr [esi]
    punpcklbw mm0,mm7
    add esi,4
    movq qword ptr [edi],mm0
    add edi,8
    add eax, -1
    jnz @B

    
    jecxz @ConvDone
    movzx eax,word ptr [esi]
    mov [edi],al
    mov [edi+2],ah   
        
    @ConvDone:

    ret 
    
 AlexConvertArea ENDP



;----------------------------------------------------------------------------
; End of program
;----------------------------------------------------------------------------

end start